% This function takes the band-pass signal recorded by GE's Vivid-i system
% (Probe 3S-RS), obtains I(t) and Q(t) by demodulating it, and then adapts
% I and Q to the new grid defined by the caller
function [adapted_data_vector,t] = AdaptRealDataToGrid(data_vector,fs_original,fs_tag,fs,fc,f_cutoff,sigma,Nsamples_new,dead_zone,debug_flag)
% Input parameters
    % data_vector: recorded data from ultrasound system
    % fs_origianl: sampling rate in system
    % fs_tag: proportional to the original sampling rate (the
        % time-window tau is chosen to be 1 for convenience, so it is multiplied by tau_factor in
        % SoS_script.m. Generally fs_tag could be equal to fs_original
    % fs: the new sampling rate (again proportional by a factor of
        % tau_factor just like fs_tag
    % fc: center frequency of band-pass pulse
    % f_cutoff: cutoff frequency for lowpass filter
    % sigma: estimated width (sigma) of gaussian pulse
    % Nsamples_new: number of desired samples after interpolation
    % dead_zone: the relative part of the beginning of the signal we want
        % to discard
    % debug_flag: determines whether graphs will be plotted for debug

%% Initialize
% number of samples
M = length(data_vector);
% timescale of recorded data
t_old = (0:M-1)/fs_original; 
% number of taps for FIR filter
taps_no = 400;
% debugging flag (plots some plots)
%% Demodulate signal to obtain I(t) and Q(t)
% design fir lowpass filter:
% f_cutoff is divided by (fs_original/2)
% since in the DFT domain a signal sampled by fs_original lies between
% -fs_original/2 to fs_original/2 (since fs_original is the Nyquist rate 
% of this signal).
my_LPF = fir1(taps_no,f_cutoff/(fs_original / 2));
%figure; freqz(my_LPF);
% Create cos(2*pi*fc*t_old) for demodulation of I(t)
cos_fc = cos(2*pi*fc*t_old).'; 
% Create sin(2*pi*fc*t_old) for demodulation of Q(t)
sin_fc = sin(2*pi*fc*t_old).';
% Perform full demodulation (multiplication by cos/sin and lowpass filtering
I_t_delayed = filter(my_LPF,1,data_vector.*cos_fc);
Q_t_delayed = filter(my_LPF,1,data_vector.*sin_fc); 
% remove delay
delay = round(length(my_LPF)/2);
I_t = I_t_delayed(delay:end);
Q_t = Q_t_delayed(delay:end);
%% Plots
% Plot original data vs. I and Q
t_old_after_delay_removal = t_old(1:length(I_t));
if debug_flag
    subplot(3,1,1);
    plot(t_old,data_vector);
    subplot(3,1,2);
    plot(t_old_after_delay_removal,I_t);
    subplot(3,1,3);
    plot(t_old_after_delay_removal,Q_t);
end
figure('Name','Original signal - dead zone removed')
plot(linspace(0,1,length(t_old)),data_vector);%.*(t_old>(dead_zone*max(t_old))))
%% Adapt data to grid
% number of samples of signal
Nsamples_old = length(I_t);
% time scale before interpolation
t_before_interpolation = (0:Nsamples_old-1)/fs_tag;
% new time scale for interpolation
t = (0:Nsamples_new-1)/fs;
% interpolate data
adapted_data_vector = interp1(t_before_interpolation(:),I_t - 1j*Q_t,t,'spline'); 
% adapted_data_vector = abs(adapted_data_vector).*(t>(dead_zone*max(t))).*(t<=max(t_before_interpolation));
adapted_data_vector = adapted_data_vector.*(t>(dead_zone*max(t))).*(t<=max(t_before_interpolation));
if debug_flag
    figure; plot(t,adapted_data_vector);
end
%% Debug area
% Plot of original band-pass signal vs. I(t), Q(t) and abs(I(t) - 1j*Q(t))
% before interpolation
if debug_flag
    % before interpolation
    figure;
    plot(t_old,data_vector)
    hold all
    plot(t_old_after_delay_removal,I_t,'r','LineWidth',1.5)
    plot(t_old_after_delay_removal,Q_t,'c','LineWidth',1.5)
    plot(t_old_after_delay_removal,abs(I_t - 1j*Q_t),'m','LineWidth',1.5)
    % after interpolation
    figure;
    plot(t_old,data_vector)
    hold all
    plot(t,real(adapted_data_vector),'r','LineWidth',1.5)
    plot(t,imag(adapted_data_vector),'c','LineWidth',1.5)
    plot(t,abs(adapted_data_vector),'m','LineWidth',1.5)
end