%% Introduction
% Created by Ronen Tur, 2009

% This script applies the Sum of Sincs (SoS) filter to a stream of Gaussian Pulses
% The filter is implemented digitally using a dense grid, mimicking analog filtering

% clc
close all
clear all
%% Dialog Box asking which problem setup is desired
button = questdlg('Which problem setup would you like to simulate?','Problem Setup','Ultrasound Imaging Data','Uniformly Distributed','Randomly Distributed','Ultrasound Imaging Data');
disp('----------------------------------------------------------------------------------------------------------');
disp('----------------------------------------------------------------------------------------------------------');
disp(strcat('Setup: ',button))
if strcmp(button,'Ultrasound Imaging Data')
    type='real_data';
else
    if strcmp(button,'Uniformly Distributed')
        type='uniformly_distributed';
    else if strcmp(button,'Randomly Distributed')
            type='random';
        end
    end
end
clear button;

% % Manual setting for debug
% type='uniformly_distributed';
% type='random';
% type='real_data'; % works with K=5 oversampling_factor=2
% type='load_5G_random_for_paper_periodic_141009';


%% User Configurable Parameters
%--------------------------------
%--------------------------------
K=5; % number of Gaussians;
oversampling_factor = 2; % oversampling factor; The number of samples taken in simulation is: N=2*K*oversampling_factor+1
Gaussian_sigma = 3e-7; % Guassian width - sigma
b_k_choice = 1; % choose SoS filter coefficients: 0 - all ones; 1 - hamming window coeff.
%--------------------------------
%--------------------------------
%% Internal simulation parameters
N=2*K*oversampling_factor + 1; % calculated number of samples, must be odd!
Rmax = 0.16; % Depth of standard echocardiographic scanning
c = 1540; % speed of sound in tissue
tau=1;    % tau is chosen to be one for convenience, tau_factor normalizes to the real world parameters
tau_factor = 2*Rmax/c/tau; 
if strcmp(type,'real_data')
    tau_factor=tau_factor/2; % Changed this in order to fit the fact there are half the samples in this setup (~4000 instead of ~8000)
end

% Params for regular vector
fs_original = 40e6;
fc=3.4235E+6; % for real data this is the correct center frequency

% fc=2e6; % used for bandpass filtering


B=N/tau; % sinc (LPF) bandwidth;
T=1/B; % Sampling period
M=floor(B*tau/2);

interp_factor = 1; % Allows creation of a more dense grid if desired
fs_tag = interp_factor*fs_original*tau_factor; % Initial sampling frequency of data from the system
%fs = fs_tag;
fs = floor(fs_tag/(N/tau))*(N/tau); % rounded sampling frequency
Nsamples = ceil(tau*fs); % no. of samples

sigma = Gaussian_sigma/tau_factor;
filter_length_one_side = 30/B*fs;

sigma_analog_noise = 0; % sigma of noise added to the signal itself
digital_SNR = 1e15; % SNR of samples after adding white gaussian noise to the samples

% Choice of coefficients b_k of SoS filter
if (b_k_choice == 1)
    b_k = hamming(2*M+1);
else 
    b_k = ones(2*M+1,1);
end

plot_flag = 0; % Toggles internal debug plotting of called functions
bandpass_flag = 0; % Flag which toggles bandpass filtering
%% Create data according to problem setup
if strcmp(type,'real_data')
    % Vector from the 12.01.2010
    load 'Master_Phantom240909_pins_positive_pulse_El31_Vec39.mat';
    data_vector = vec39;
    f_cutoff = 1.7e6; % Desired cutoff frequency of LPF
    debug_flag = 0;
    dead_zone_factor = 0; % discard the dead zone (first tenth) of the signal
    % The next row adapts the original data to the newly defined sampling
    % frequency fs.
    [x_t_before_thresh,t] = AdaptRealDataToGrid(data_vector,fs_original,fs_tag,fs,fc,f_cutoff,sigma,Nsamples,dead_zone_factor,debug_flag);
    % normalize x_t_before_thresh
    x_t_before_thresh = x_t_before_thresh/max(x_t_before_thresh);
    % hard threshold
    threshold = 0.1; % threshold for hard thresholding of real data. in units of max(abs(x_t))
    x_t = x_t_before_thresh.*(abs(x_t_before_thresh)>threshold*max(abs(x_t_before_thresh)));
    tk=0;
    xk=0;
    tk_known_flag = 0;
    times=0;
else
    calibrate_flag = 0; % in SoS solution no calibration needed!
    % The next row creates synthetic data vector according to given params
    [x_t,t,tk,xk,times,x_t_without_noise] = CreateSyntheticData(K,tau,B,fs,type,calibrate_flag,sigma,bandpass_flag,fc,sigma_analog_noise);
    tk_known_flag = 1;
end
%% Estimate tk and xk
button = questdlg('Would you like to apply the periodic or aperiodic solution?','Method Choice','Aperiodic','Periodic','Aperiodic');
disp(strcat('Method: ',button))
if strcmp(button,'Periodic')
    [tk_FRI,tk_FRI_Cadzow,xk_est]=SoS_Idea_Periodic_Func(x_t.',K,tau,B,fs,Nsamples,t,times,bandpass_flag,fc,N,T,sigma,tk,xk,tk_known_flag,sigma_analog_noise,digital_SNR,b_k,plot_flag);
else
    [tk_FRI,tk_FRI_Cadzow,xk_est]=SoS_Idea_Aperiodic_Func(x_t.',K,tau,B,fs,Nsamples,t,times,bandpass_flag,fc,N,T,sigma,tk,xk,tk_known_flag,sigma_analog_noise,digital_SNR,b_k,plot_flag);
end
%% Plot original signal (before thresholding) vs. estimated one, real data
% case only
if strcmp(type,'real_data')
    tk_est = tk_FRI_Cadzow;
    times_est = repmat(t.',1,K) - repmat(tk_est.',length(t),1);
    x_t_estimated = exp( - ( (times_est).^2 / (2*sigma^2) ) )*real(xk_est);
    % x_t_estimated = exp( - ( (times_est).^2 / (2*sigma^2) ) ).*cos(2*pi*fc*times)*real(xk_est);
    
    figure('Name','Original signal x_t vs. estimated signal');
    plot(t,real(x_t_before_thresh),'LineWidth',2.5);
    hold all
    plot(t,x_t_estimated,'--r','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',1)
    legend('Original Signal','Reconstructed Signal','Location','SouthEast');
    xlabel('time [units of \tau]'); ylabel('Amplitude');
    
%     figure('Name','Original signal x_t + estimated positions of gaussians');
%     plot(t,real(x_t_before_thresh),'LineWidth',1.5)
%     hold all
%     plot(tk_est,real(xk_est),'o','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)
%     legend('Original Signal','Estimated times and amplitudes')%,'Location','SouthOutside');
%     xlabel('time [units of \tau]'); ylabel('Amplitude');
    
    figure('Name','Original signal x_t + estimated positions of gaussians');
    plot(t,real(x_t_before_thresh),'LineWidth',1.5)
    hold all
    stem(tk_est,real(xk_est),'o','LineWidth',0.1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)
    % legend('Original Signal','Estimated times and amplitudes')%,'Location','SouthOutside');
    legend('Original Signal','Estimated parameters')%,'Location','SouthOutside');
    xlabel('time [units of \tau]'); ylabel('Amplitude');
%     figure(14)
end
disp(' ')
disp('Simulation terminated succesfully')
disp('----------------------------------------------------------------------------------------------------------');
%% Save workspace to file
% save(strcat('Ultrasound_RealData_K',int2str(K),'_oversampling_factor',int2str(oversampling_factor),'_THR',int2str(threshold*100),'percent'))


